﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace UsbDetectorCs {
   public partial class Form1 : Form {

      public Form1() { InitializeComponent(); }

      private const int WM_DEVICECHANGE = 0x219;
      private const int DBT_DEVICEARRIVAL = 0x8000;
      private const int DBT_DEVICEREMOVECOMPLETE = 0x8004;

      private int _Counter = 0;
      protected override void WndProc(ref Message msg) {
         base.WndProc(ref msg);
         if (msg.Msg != WM_DEVICECHANGE) return;
         var remark = "";
         switch (msg.WParam.ToInt32()) {
         case DBT_DEVICEARRIVAL:
            remark = "add drive: " + DEV_BROADCAST_VOLUME.FromPointer(msg.LParam).GetDriveChar();
            break;
         case DBT_DEVICEREMOVECOMPLETE:
            remark = "remove drive: " + DEV_BROADCAST_VOLUME.FromPointer(msg.LParam).GetDriveChar();
            break;
         }
         Debug.WriteLine(msg.ToString());
         WriteLog(msg.ToString(), remark);
      }

      /// <summary> fügt msg und remark als eine numerierte neue Zeile einer Richtextbox an (die maximal 30 Zeilen anzeigt) </summary>
      private void WriteLog(string msg, string remark) {
         //zb: msg=0x219 (WM_DEVICECHANGE) hwnd=0x950d8e wparam=0x7 lparam=0x0 result=0x1
         if (!string.IsNullOrEmpty(remark)) remark = string.Concat(" remark=\"", remark, "\"");
         var line = string.Format("{0:000}: {1} {2}",_Counter, msg, remark);
         var lines = RichTextBox1.Lines;
         RichTextBox1.Lines = lines.Concat(new string[] { line }).Skip(lines.Length - 30).ToArray();
         _Counter += 1;
      }
   }

   // Struct for parameters of the WM_DEVICECHANGE message
   [StructLayout(LayoutKind.Sequential)]
   public struct DEV_BROADCAST_VOLUME {
      public int dbcv_size;
      public int dbcv_devicetype;
      public int dbcv_reserved;

      public int dbcv_unitmask;
      public char GetDriveChar() {
         //die Position des in dbcv_unitmask gesetzten Bits entspricht dem Laufwerks-Buchstaben
         for (var i = 0; i <= 31; i++) {
            if ((1 << i & dbcv_unitmask) != 0) return Convert.ToChar(65 + i);
         }
         return '?';
      }

      public static DEV_BROADCAST_VOLUME FromPointer(IntPtr ptr) {
         return (DEV_BROADCAST_VOLUME)Marshal.PtrToStructure(ptr, typeof(DEV_BROADCAST_VOLUME));
      }
   }

}
